==> rewrite-github-actions-3.14.0-sources.jar: org/openrewrite/github/FindMissingTimeout.java <==
/*
 * Copyright 2025 the original author or authors.
 * <p>
 * Licensed under the Moderne Source Available License (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * https://docs.moderne.io/licensing/moderne-source-available-license
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.openrewrite.github;

import org.openrewrite.*;
import org.openrewrite.marker.SearchResult;
import org.openrewrite.yaml.JsonPathMatcher;
import org.openrewrite.yaml.YamlIsoVisitor;
import org.openrewrite.yaml.search.FindKey;
import org.openrewrite.yaml.tree.Yaml;

public class FindMissingTimeout extends Recipe {
    @Override
    public String getDisplayName() {
        return "Find jobs missing timeout";
    }

    @Override
    public String getDescription() {
        return "Find GitHub Actions jobs missing a timeout.";
    }

    @Override
    public TreeVisitor<?, ExecutionContext> getVisitor() {
        return Preconditions.check(new FindSourceFiles(".github/workflows/*.yml"), new YamlIsoVisitor<ExecutionContext>() {
            @Override
            public Yaml.Mapping.Entry visitMappingEntry(Yaml.Mapping.Entry entry, ExecutionContext ctx) {
                Yaml.Mapping.Entry job = super.visitMappingEntry(entry, ctx);
                if (new JsonPathMatcher("$.jobs.*.*").matches(getCursor()) &&
                        FindKey.find(job, "$..timeout-minutes").isEmpty()) {
                    return SearchResult.found(job, "missing: $.jobs.*.timeout-minutes");
                }
                return job;
            }
        });
    }
}

==> rewrite-github-actions-3.15.0-sources.jar: org/openrewrite/github/FindMissingTimeout.java <==
/*
 * Copyright 2025 the original author or authors.
 * <p>
 * Licensed under the Moderne Source Available License (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * https://docs.moderne.io/licensing/moderne-source-available-license
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.openrewrite.github;

import org.openrewrite.ExecutionContext;
import org.openrewrite.Preconditions;
import org.openrewrite.Recipe;
import org.openrewrite.TreeVisitor;
import org.openrewrite.marker.SearchResult;
import org.openrewrite.yaml.JsonPathMatcher;
import org.openrewrite.yaml.YamlIsoVisitor;
import org.openrewrite.yaml.search.FindKey;
import org.openrewrite.yaml.tree.Yaml;

public class FindMissingTimeout extends Recipe {
    @Override
    public String getDisplayName() {
        return "Find jobs missing timeout";
    }

    @Override
    public String getDescription() {
        return "Find GitHub Actions jobs missing a timeout.";
    }

    @Override
    public TreeVisitor<?, ExecutionContext> getVisitor() {
        return Preconditions.check(new IsGitHubActionsWorkflow(), new YamlIsoVisitor<ExecutionContext>() {
            @Override
            public Yaml.Mapping.Entry visitMappingEntry(Yaml.Mapping.Entry entry, ExecutionContext ctx) {
                Yaml.Mapping.Entry job = super.visitMappingEntry(entry, ctx);
                if (new JsonPathMatcher("$.jobs.*.*").matches(getCursor()) &&
                        FindKey.find(job, "$..timeout-minutes").isEmpty()) {
                    return SearchResult.found(job, "missing: $.jobs.*.timeout-minutes");
                }
                return job;
            }
        });
    }
}
